@@ -1,7 +1,10 @@
 use crate::app;
 use crate::app::HelpMenuLine;
 use crate::app::{Node, ResolvedNode};
+use crate::config::BlockConfig;
+use crate::config::Layout;
 use handlebars::Handlebars;
+use indexmap::IndexSet;
 use lazy_static::lazy_static;
 use serde::{Deserialize, Serialize};
 use std::cmp::Ordering;
@@ -9,10 +12,10 @@ use std::collections::HashMap;
 use std::env;
 use tui::backend::Backend;
 use tui::layout::Rect;
-use tui::layout::{Constraint as TuiConstraint, Direction, Layout};
-use tui::style::{Color, Modifier, Style as TuiStyle};
+use tui::layout::{Constraint as TuiConstraint, Direction, Layout as TuiLayout};
+use tui::style::{Color, Modifier as TuiModifier, Style as TuiStyle};
 use tui::text::{Span, Spans};
-use tui::widgets::{Block, Borders, Cell, List, ListItem, Paragraph, Row, Table};
+use tui::widgets::{Block, Borders as TuiBorders, Cell, List, ListItem, Paragraph, Row, Table};
 use tui::Frame;
 
 lazy_static! {
@@ -20,36 +23,84 @@ lazy_static! {
     pub static ref DEFAULT_STYLE: TuiStyle = TuiStyle::default();
 }
 
-#[derive(Debug, Copy, Clone, Default, PartialEq, Serialize, Deserialize)]
+#[derive(Debug, Clone, Copy, Hash, PartialEq, Eq, PartialOrd, Serialize, Deserialize)]
+#[serde(deny_unknown_fields)]
+pub enum Border {
+    Top,
+    Right,
+    Bottom,
+    Left,
+}
+
+impl Border {
+    pub fn bits(self) -> u32 {
+        match self {
+            Self::Top => TuiBorders::TOP.bits(),
+            Self::Right => TuiBorders::RIGHT.bits(),
+            Self::Bottom => TuiBorders::BOTTOM.bits(),
+            Self::Left => TuiBorders::LEFT.bits(),
+        }
+    }
+}
+
+#[derive(Debug, Copy, Clone, Hash, PartialEq, Eq, PartialOrd, Serialize, Deserialize)]
+#[serde(deny_unknown_fields)]
+pub enum Modifier {
+    Bold,
+    Dim,
+    Italic,
+    Underlined,
+    SlowBlink,
+    RapidBlink,
+    Reversed,
+    Hidden,
+    CrossedOut,
+}
+
+impl Modifier {
+    pub fn bits(self) -> u16 {
+        match self {
+            Self::Bold => TuiModifier::BOLD.bits(),
+            Self::Dim => TuiModifier::DIM.bits(),
+            Self::Italic => TuiModifier::ITALIC.bits(),
+            Self::Underlined => TuiModifier::UNDERLINED.bits(),
+            Self::SlowBlink => TuiModifier::SLOW_BLINK.bits(),
+            Self::RapidBlink => TuiModifier::RAPID_BLINK.bits(),
+            Self::Reversed => TuiModifier::REVERSED.bits(),
+            Self::Hidden => TuiModifier::HIDDEN.bits(),
+            Self::CrossedOut => TuiModifier::CROSSED_OUT.bits(),
+        }
+    }
+}
+
+#[derive(Debug, Clone, Default, Serialize, Deserialize)]
 #[serde(deny_unknown_fields)]
 pub struct Style {
     fg: Option<Color>,
     bg: Option<Color>,
-    add_modifier: Option<Modifier>,
-    sub_modifier: Option<Modifier>,
+    add_modifiers: Option<IndexSet<Modifier>>,
+    sub_modifiers: Option<IndexSet<Modifier>>,
+}
+
+impl PartialEq for Style {
+    fn eq(&self, other: &Self) -> bool {
+        self.fg == other.fg
+            && self.bg == other.bg
+            && self.add_modifiers == other.add_modifiers
+            && self.sub_modifiers == other.sub_modifiers
+    }
 }
 
 impl Style {
     pub fn extend(mut self, other: Self) -> Self {
         self.fg = other.fg.or(self.fg);
         self.bg = other.bg.or(self.bg);
-        self.add_modifier = other.add_modifier.or(self.add_modifier);
-        self.sub_modifier = other.sub_modifier.or(self.sub_modifier);
+        self.add_modifiers = other.add_modifiers.or(self.add_modifiers);
+        self.sub_modifiers = other.sub_modifiers.or(self.sub_modifiers);
         self
     }
 }
 
-impl From<TuiStyle> for Style {
-    fn from(s: TuiStyle) -> Self {
-        Self {
-            fg: s.fg,
-            bg: s.bg,
-            add_modifier: Some(s.add_modifier),
-            sub_modifier: Some(s.sub_modifier),
-        }
-    }
-}
-
 impl Into<TuiStyle> for Style {
     fn into(self) -> TuiStyle {
         if *NO_COLOR {
@@ -58,8 +109,23 @@ impl Into<TuiStyle> for Style {
             TuiStyle {
                 fg: self.fg,
                 bg: self.bg,
-                add_modifier: self.add_modifier.unwrap_or_else(Modifier::empty),
-                sub_modifier: self.sub_modifier.unwrap_or_else(Modifier::empty),
+                add_modifier: TuiModifier::from_bits_truncate(
+                    self.add_modifiers
+                        .unwrap_or_default()
+                        .into_iter()
+                        .map(|m| m.bits())
+                        .reduce(|a, b| (a ^ b))
+                        .unwrap_or_else(|| TuiModifier::empty().bits()),
+                ),
+
+                sub_modifier: TuiModifier::from_bits_truncate(
+                    self.sub_modifiers
+                        .unwrap_or_default()
+                        .into_iter()
+                        .map(|m| m.bits())
+                        .reduce(|a, b| (a ^ b))
+                        .unwrap_or_else(|| TuiModifier::empty().bits()),
+                ),
             }
         }
     }
@@ -167,10 +233,36 @@ impl NodeUiMetadata {
     }
 }
 
-fn draw_table<B: Backend>(f: &mut Frame<B>, rect: Rect, app: &app::App, hb: &Handlebars) {
-    let config = app.config().to_owned();
-    let header_height = config.general().table().header().height().unwrap_or(1);
-    let height: usize = (rect.height.max(header_height + 2) - (header_height + 2)).into();
+fn block<'a>(config: BlockConfig, default_title: String) -> Block<'a> {
+    Block::default()
+        .borders(TuiBorders::from_bits_truncate(
+            config
+                .borders()
+                .clone()
+                .unwrap_or_default()
+                .iter()
+                .map(|b| b.bits())
+                .reduce(|a, b| (a ^ b))
+                .unwrap_or_else(|| TuiBorders::NONE.bits()),
+        ))
+        .title(Span::styled(
+            config.title().format().clone().unwrap_or(default_title),
+            config.title().style().clone().into(),
+        ))
+        .style(config.style().clone().into())
+}
+
+fn draw_table<B: Backend>(
+    config: BlockConfig,
+    f: &mut Frame<B>,
+    screen_size: Rect,
+    layout_size: Rect,
+    app: &app::App,
+    hb: &Handlebars,
+) {
+    let app_config = app.config().to_owned();
+    let header_height = app_config.general().table().header().height().unwrap_or(1);
+    let height: usize = (layout_size.height.max(header_height + 2) - (header_height + 2)).into();
 
     let rows = app
         .directory_buffer()
@@ -189,7 +281,7 @@ fn draw_table<B: Backend>(f: &mut Frame<B>, rect: Rect, app: &app::App, hb: &Han
                     let is_first = index == 0;
                     let is_last = index == dir.total().max(1) - 1;
 
-                    let tree = config
+                    let tree = app_config
                         .general()
                         .table()
                         .tree()
@@ -205,19 +297,24 @@ fn draw_table<B: Backend>(f: &mut Frame<B>, rect: Rect, app: &app::App, hb: &Han
                         })
                         .unwrap_or_default();
 
-                    let node_type = config
+                    let node_type = app_config
                         .node_types()
                         .special()
                         .get(node.relative_path())
-                        .or_else(|| config.node_types().extension().get(node.extension()))
-                        .or_else(|| config.node_types().mime_essence().get(node.mime_essence()))
+                        .or_else(|| app_config.node_types().extension().get(node.extension()))
+                        .or_else(|| {
+                            app_config
+                                .node_types()
+                                .mime_essence()
+                                .get(node.mime_essence())
+                        })
                         .unwrap_or_else(|| {
                             if node.is_symlink() {
-                                &config.node_types().symlink()
+                                &app_config.node_types().symlink()
                             } else if node.is_dir() {
-                                &config.node_types().directory()
+                                &app_config.node_types().directory()
                             } else {
-                                &config.node_types().file()
+                                &app_config.node_types().file()
                             }
                         });
 
@@ -229,26 +326,26 @@ fn draw_table<B: Backend>(f: &mut Frame<B>, rect: Rect, app: &app::App, hb: &Han
                         };
 
                     let (mut prefix, mut suffix, mut style) = {
-                        let ui = config.general().default_ui().clone();
+                        let ui = app_config.general().default_ui().clone();
                         (
                             ui.prefix().clone(),
                             ui.suffix().clone(),
-                            ui.style().extend(node_type.style()),
+                            ui.style().clone().extend(node_type.style().clone()),
                         )
                     };
 
                     if is_selected {
-                        let ui = config.general().selection_ui().clone();
+                        let ui = app_config.general().selection_ui().clone();
                         prefix = ui.prefix().clone().or(prefix);
                         suffix = ui.suffix().clone().or(suffix);
-                        style = style.extend(ui.style());
+                        style = style.extend(ui.style().clone());
                     };
 
                     if is_focused {
-                        let ui = config.general().focus_ui().clone();
+                        let ui = app_config.general().focus_ui().clone();
                         prefix = ui.prefix().clone().or(prefix);
                         suffix = ui.suffix().clone().or(suffix);
-                        style = style.extend(ui.style());
+                        style = style.extend(ui.style().clone());
                     };
 
                     let meta = NodeUiMetadata::new(
@@ -280,30 +377,41 @@ fn draw_table<B: Backend>(f: &mut Frame<B>, rect: Rect, app: &app::App, hb: &Han
         })
         .unwrap_or_default();
 
-    let table_constraints: Vec<TuiConstraint> = config
+    let table_constraints: Vec<TuiConstraint> = app_config
         .general()
         .table()
         .col_widths()
         .clone()
         .unwrap_or_default()
         .into_iter()
-        .map(|c| c.into())
+        .map(|c| c.to_tui(screen_size, layout_size))
         .collect();
 
     let table = Table::new(rows)
         .widths(&table_constraints)
-        .style(config.general().table().style().into())
-        .highlight_style(config.general().focus_ui().style().into())
-        .column_spacing(config.general().table().col_spacing().unwrap_or_default())
-        .block(Block::default().borders(Borders::ALL).title(format!(
-            " {} ({}) ",
-            app.pwd(),
-            app.directory_buffer().map(|d| d.total()).unwrap_or_default()
-        )));
+        .style(app_config.general().table().style().clone().into())
+        .highlight_style(app_config.general().focus_ui().style().clone().into())
+        .column_spacing(
+            app_config
+                .general()
+                .table()
+                .col_spacing()
+                .unwrap_or_default(),
+        )
+        .block(block(
+            config,
+            format!(
+                " {} ({}) ",
+                app.pwd(),
+                app.directory_buffer()
+                    .map(|d| d.total())
+                    .unwrap_or_default()
+            ),
+        ));
 
     let table = table.clone().header(
         Row::new(
-            config
+            app_config
                 .general()
                 .table()
                 .header()
@@ -315,18 +423,25 @@ fn draw_table<B: Backend>(f: &mut Frame<B>, rect: Rect, app: &app::App, hb: &Han
                 .collect::<Vec<Cell>>(),
         )
         .height(header_height)
-        .style(config.general().table().header().style().into()),
+        .style(app_config.general().table().header().style().clone().into()),
     );
 
-    f.render_widget(table, rect);
+    f.render_widget(table, layout_size);
 }
 
-fn draw_selection<B: Backend>(f: &mut Frame<B>, rect: Rect, app: &app::App, _: &Handlebars) {
+fn draw_selection<B: Backend>(
+    config: BlockConfig,
+    f: &mut Frame<B>,
+    _screen_size: Rect,
+    layout_size: Rect,
+    app: &app::App,
+    _: &Handlebars,
+) {
     let selection: Vec<ListItem> = app
         .selection()
         .iter()
         .rev()
-        .take((rect.height.max(2) - 2).into())
+        .take((layout_size.height.max(2) - 2).into())
         .rev()
         .map(|n| n.absolute_path().clone())
         .map(ListItem::new)
@@ -335,16 +450,20 @@ fn draw_selection<B: Backend>(f: &mut Frame<B>, rect: Rect, app: &app::App, _: &
     let selection_count = selection.len();
 
     // Selected items
-    let selection_list = List::new(selection).block(
-        Block::default()
-            .borders(Borders::ALL)
-            .title(format!(" Selection ({}) ", selection_count)),
-    );
+    let selection_list =
+        List::new(selection).block(block(config, format!(" Selection ({}) ", selection_count)));
 
-    f.render_widget(selection_list, rect);
+    f.render_widget(selection_list, layout_size);
 }
 
-fn draw_help_menu<B: Backend>(f: &mut Frame<B>, rect: Rect, app: &app::App, _: &Handlebars) {
+fn draw_help_menu<B: Backend>(
+    config: BlockConfig,
+    f: &mut Frame<B>,
+    _screen_size: Rect,
+    layout_size: Rect,
+    app: &app::App,
+    _: &Handlebars,
+) {
     let help_menu_rows = app
         .mode()
         .help_menu()
@@ -373,20 +492,26 @@ fn draw_help_menu<B: Backend>(f: &mut Frame<B>, rect: Rect, app: &app::App, _: &
     };
 
     let help_menu = Table::new(help_menu_rows)
-        .block(Block::default().borders(Borders::ALL).title(format!(
-            " Help [{}{}] ",
-            &app.mode().name(),
-            read_only_indicator
-        )))
+        .block(block(
+            config,
+            format!(" Help [{}{}] ", &app.mode().name(), read_only_indicator),
+        ))
         .widths(&[
             TuiConstraint::Percentage(20),
             TuiConstraint::Percentage(20),
             TuiConstraint::Percentage(60),
         ]);
-    f.render_widget(help_menu, rect);
+    f.render_widget(help_menu, layout_size);
 }
 
-fn draw_input_buffer<B: Backend>(f: &mut Frame<B>, rect: Rect, app: &app::App, _: &Handlebars) {
+fn draw_input_buffer<B: Backend>(
+    config: BlockConfig,
+    f: &mut Frame<B>,
+    _screen_size: Rect,
+    layout_size: Rect,
+    app: &app::App,
+    _: &Handlebars,
+) {
     let input_buf = Paragraph::new(Spans::from(vec![
         Span::styled(
             app.config()
@@ -395,7 +520,7 @@ fn draw_input_buffer<B: Backend>(f: &mut Frame<B>, rect: Rect, app: &app::App, _
                 .format()
                 .clone()
                 .unwrap_or_default(),
-            app.config().general().prompt().style().into(),
+            app.config().general().prompt().style().clone().into(),
         ),
         Span::raw(app.input_buffer().unwrap_or_else(|| "".into())),
         Span::styled(
@@ -405,14 +530,21 @@ fn draw_input_buffer<B: Backend>(f: &mut Frame<B>, rect: Rect, app: &app::App, _
                 .format()
                 .clone()
                 .unwrap_or_default(),
-            app.config().general().cursor().style().into(),
+            app.config().general().cursor().style().clone().into(),
         ),
     ]))
-    .block(Block::default().borders(Borders::ALL).title(" Input "));
-    f.render_widget(input_buf, rect);
+    .block(block(config, " Input ".into()));
+    f.render_widget(input_buf, layout_size);
 }
 
-fn draw_sort_n_filter_by<B: Backend>(f: &mut Frame<B>, rect: Rect, app: &app::App, _: &Handlebars) {
+fn draw_sort_n_filter<B: Backend>(
+    config: BlockConfig,
+    f: &mut Frame<B>,
+    _screen_size: Rect,
+    layout_size: Rect,
+    app: &app::App,
+    _: &Handlebars,
+) {
     let ui = app.config().general().sort_and_filter_ui().clone();
     let filter_by = app.explorer_config().filters();
     let sort_by = app.explorer_config().sorters();
@@ -422,7 +554,11 @@ fn draw_sort_n_filter_by<B: Backend>(f: &mut Frame<B>, rect: Rect, app: &app::Ap
             .format()
             .to_owned()
             .unwrap_or_default(),
-        ui.sort_direction_identifiers().forward().style().into(),
+        ui.sort_direction_identifiers()
+            .forward()
+            .style()
+            .clone()
+            .into(),
     );
 
     let reverse = Span::styled(
@@ -431,7 +567,11 @@ fn draw_sort_n_filter_by<B: Backend>(f: &mut Frame<B>, rect: Rect, app: &app::Ap
             .format()
             .to_owned()
             .unwrap_or_default(),
-        ui.sort_direction_identifiers().reverse().style().into(),
+        ui.sort_direction_identifiers()
+            .reverse()
+            .style()
+            .clone()
+            .into(),
     );
 
     let mut spans = filter_by
@@ -441,7 +581,10 @@ fn draw_sort_n_filter_by<B: Backend>(f: &mut Frame<B>, rect: Rect, app: &app::Ap
                 .get(&f.filter())
                 .map(|u| {
                     (
-                        Span::styled(u.format().to_owned().unwrap_or_default(), u.style().into()),
+                        Span::styled(
+                            u.format().to_owned().unwrap_or_default(),
+                            u.style().clone().into(),
+                        ),
                         Span::raw(f.input().clone()),
                     )
                 })
@@ -458,7 +601,10 @@ fn draw_sort_n_filter_by<B: Backend>(f: &mut Frame<B>, rect: Rect, app: &app::Ap
                 .get(&s.sorter())
                 .map(|u| {
                     (
-                        Span::styled(u.format().to_owned().unwrap_or_default(), u.style().into()),
+                        Span::styled(
+                            u.format().to_owned().unwrap_or_default(),
+                            u.style().clone().into(),
+                        ),
                         direction.clone(),
                     )
                 })
@@ -466,22 +612,30 @@ fn draw_sort_n_filter_by<B: Backend>(f: &mut Frame<B>, rect: Rect, app: &app::Ap
         }))
         .zip(std::iter::repeat(Span::styled(
             ui.separator().format().to_owned().unwrap_or_default(),
-            ui.separator().style().into(),
+            ui.separator().style().clone().into(),
         )))
         .map(|((a, b), c)| vec![a, b, c])
         .flatten()
         .collect::<Vec<Span>>();
     spans.pop();
 
-    let p = Paragraph::new(Spans::from(spans)).block(Block::default().borders(Borders::ALL).title(
+    let p = Paragraph::new(Spans::from(spans)).block(block(
+        config,
         format!(" Sort & filter ({}) ", filter_by.len() + sort_by.len()),
     ));
 
-    f.render_widget(p, rect);
+    f.render_widget(p, layout_size);
 }
 
-fn draw_logs<B: Backend>(f: &mut Frame<B>, rect: Rect, app: &app::App, _: &Handlebars) {
-    let config = app.config().general().logs().clone();
+fn draw_logs<B: Backend>(
+    config: BlockConfig,
+    f: &mut Frame<B>,
+    _screen_size: Rect,
+    layout_size: Rect,
+    app: &app::App,
+    _: &Handlebars,
+) {
+    let logs_config = app.config().general().logs().clone();
     let logs = app
         .logs()
         .iter()
@@ -494,73 +648,143 @@ fn draw_logs<B: Backend>(f: &mut Frame<B>, rect: Rect, app: &app::App, _: &Handl
                 app::LogLevel::Info => ListItem::new(format!(
                     "{} | {} | {}",
                     &time,
-                    &config.info().format().to_owned().unwrap_or_default(),
+                    &logs_config.info().format().to_owned().unwrap_or_default(),
                     l.message()
                 ))
-                .style(config.info().style().into()),
+                .style(logs_config.info().style().clone().into()),
                 app::LogLevel::Success => ListItem::new(format!(
                     "{} | {} | {}",
                     &time,
-                    &config.success().format().to_owned().unwrap_or_default(),
+                    &logs_config
+                        .success()
+                        .format()
+                        .to_owned()
+                        .unwrap_or_default(),
                     l.message()
                 ))
-                .style(config.success().style().into()),
+                .style(logs_config.success().style().clone().into()),
                 app::LogLevel::Error => ListItem::new(format!(
                     "{} | {} | {}",
                     &time,
-                    &config.error().format().to_owned().unwrap_or_default(),
+                    &logs_config.error().format().to_owned().unwrap_or_default(),
                     l.message()
                 ))
-                .style(config.error().style().into()),
+                .style(logs_config.error().style().clone().into()),
             }
         })
         .collect::<Vec<ListItem>>();
 
-    let logs_list = List::new(logs).block(
-        Block::default()
-            .borders(Borders::ALL)
-            .title(format!(" Logs ({}) ", app.logs().len())),
-    );
+    let logs_list = List::new(logs).block(block(config, format!(" Logs ({}) ", app.logs().len())));
 
-    f.render_widget(logs_list, rect);
+    f.render_widget(logs_list, layout_size);
+}
+
+pub fn draw_nothing<B: Backend>(
+    config: BlockConfig,
+    f: &mut Frame<B>,
+    _screen_size: Rect,
+    layout_size: Rect,
+    _app: &app::App,
+    _hb: &Handlebars,
+) {
+    let nothing = Paragraph::new("").block(block(config, "".into()));
+    f.render_widget(nothing, layout_size);
+}
+
+pub fn draw_layout<B: Backend>(
+    layout: Layout,
+    f: &mut Frame<B>,
+    screen_size: Rect,
+    layout_size: Rect,
+    app: &app::App,
+    hb: &Handlebars,
+) {
+    match layout {
+        Layout::Nothing(config) => draw_nothing(config, f, screen_size, layout_size, app, hb),
+        Layout::Table(config) => draw_table(config, f, screen_size, layout_size, app, hb),
+        Layout::SortAndFilter(config) => {
+            draw_sort_n_filter(config, f, screen_size, layout_size, app, hb)
+        }
+        Layout::HelpMenu(config) => draw_help_menu(config, f, screen_size, layout_size, app, hb),
+        Layout::Selection(config) => draw_selection(config, f, screen_size, layout_size, app, hb),
+        Layout::InputAndLogs(config) => {
+            if app.input_buffer().is_some() {
+                draw_input_buffer(config, f, screen_size, layout_size, app, hb);
+            } else {
+                draw_logs(config, f, screen_size, layout_size, app, hb);
+            };
+        }
+        Layout::Horizontal { config, splits } => {
+            let chunks = TuiLayout::default()
+                .direction(Direction::Horizontal)
+                .constraints(
+                    config
+                        .constraints()
+                        .clone()
+                        .unwrap_or_default()
+                        .iter()
+                        .map(|c| c.to_tui(screen_size, layout_size))
+                        .collect::<Vec<TuiConstraint>>(),
+                )
+                .horizontal_margin(
+                    config
+                        .horizontal_margin()
+                        .or_else(|| config.margin())
+                        .unwrap_or_default(),
+                )
+                .vertical_margin(
+                    config
+                        .vertical_margin()
+                        .or_else(|| config.margin())
+                        .unwrap_or_default(),
+                )
+                .split(layout_size);
+
+            splits
+                .into_iter()
+                .zip(chunks.into_iter())
+                .for_each(|(split, chunk)| draw_layout(split, f, screen_size, chunk, app, hb));
+        }
+
+        Layout::Vertical { config, splits } => {
+            let chunks = TuiLayout::default()
+                .direction(Direction::Vertical)
+                .constraints(
+                    config
+                        .constraints()
+                        .clone()
+                        .unwrap_or_default()
+                        .iter()
+                        .map(|c| c.to_tui(screen_size, layout_size))
+                        .collect::<Vec<TuiConstraint>>(),
+                )
+                .horizontal_margin(
+                    config
+                        .horizontal_margin()
+                        .or_else(|| config.margin())
+                        .unwrap_or_default(),
+                )
+                .vertical_margin(
+                    config
+                        .vertical_margin()
+                        .or_else(|| config.margin())
+                        .unwrap_or_default(),
+                )
+                .split(layout_size);
+
+            splits
+                .into_iter()
+                .zip(chunks.into_iter())
+                .for_each(|(split, chunk)| draw_layout(split, f, screen_size, chunk, app, hb));
+        }
+    }
 }
 
 pub fn draw<B: Backend>(f: &mut Frame<B>, app: &app::App, hb: &Handlebars) {
-    let rect = f.size();
+    let screen_size = f.size();
+    let layout = app.layout().clone();
 
-    let chunks = Layout::default()
-        .direction(Direction::Horizontal)
-        .constraints([TuiConstraint::Percentage(70), TuiConstraint::Percentage(30)].as_ref())
-        .split(rect);
-
-    let left_chunks = Layout::default()
-        .direction(Direction::Vertical)
-        .constraints(
-            [
-                TuiConstraint::Length(3),
-                TuiConstraint::Length(rect.height - 6),
-                TuiConstraint::Length(3),
-            ]
-            .as_ref(),
-        )
-        .split(chunks[0]);
-
-    draw_sort_n_filter_by(f, left_chunks[0], app, hb);
-    draw_table(f, left_chunks[1], app, hb);
-
-    if app.input_buffer().is_some() {
-        draw_input_buffer(f, left_chunks[2], app, hb);
-    } else {
-        draw_logs(f, left_chunks[2], app, hb);
-    };
-
-    let right_chunks = Layout::default()
-        .direction(Direction::Vertical)
-        .constraints([TuiConstraint::Percentage(50), TuiConstraint::Percentage(50)].as_ref())
-        .split(chunks[1]);
-
-    draw_selection(f, right_chunks[0], app, hb);
-    draw_help_menu(f, right_chunks[1], app, hb);
+    draw_layout(layout, f, screen_size, screen_size, app, hb);
 }
 
 #[cfg(test)]
@@ -568,29 +792,34 @@ mod test {
     use super::*;
     use crate::config;
     use tui::style::Color;
-    use tui::style::Modifier;
+
+    fn modifier(m: Modifier) -> Option<IndexSet<Modifier>> {
+        let mut x = IndexSet::new();
+        x.insert(m);
+        Some(x)
+    }
 
     #[test]
     fn test_extend_style() {
         let a = Style {
             fg: Some(Color::Red),
             bg: None,
-            add_modifier: Some(Modifier::BOLD),
-            sub_modifier: None,
+            add_modifiers: modifier(Modifier::Bold),
+            sub_modifiers: None,
         };
 
         let b = Style {
             fg: None,
             bg: Some(Color::Blue),
-            add_modifier: None,
-            sub_modifier: Some(Modifier::DIM),
+            add_modifiers: None,
+            sub_modifiers: modifier(Modifier::Dim),
         };
 
         let c = Style {
             fg: Some(Color::Cyan),
             bg: Some(Color::Magenta),
-            add_modifier: Some(Modifier::CROSSED_OUT),
-            sub_modifier: Some(Modifier::ITALIC),
+            add_modifiers: modifier(Modifier::CrossedOut),
+            sub_modifiers: modifier(Modifier::Italic),
         };
 
         assert_eq!(
@@ -598,8 +827,8 @@ mod test {
             Style {
                 fg: Some(Color::Red),
                 bg: Some(Color::Blue),
-                add_modifier: Some(Modifier::BOLD),
-                sub_modifier: Some(Modifier::DIM),
+                add_modifiers: modifier(Modifier::Bold),
+                sub_modifiers: modifier(Modifier::Dim),
             }
         );
 
@@ -608,8 +837,8 @@ mod test {
             Style {
                 fg: Some(Color::Red),
                 bg: Some(Color::Blue),
-                add_modifier: Some(Modifier::BOLD),
-                sub_modifier: Some(Modifier::DIM),
+                add_modifiers: modifier(Modifier::Bold),
+                sub_modifiers: modifier(Modifier::Dim),
             }
         );
 
@@ -618,8 +847,8 @@ mod test {
             Style {
                 fg: Some(Color::Cyan),
                 bg: Some(Color::Magenta),
-                add_modifier: Some(Modifier::CROSSED_OUT),
-                sub_modifier: Some(Modifier::ITALIC),
+                add_modifiers: modifier(Modifier::CrossedOut),
+                sub_modifiers: modifier(Modifier::Italic),
             }
         );
 
@@ -628,8 +857,8 @@ mod test {
             Style {
                 fg: Some(Color::Red),
                 bg: Some(Color::Magenta),
-                add_modifier: Some(Modifier::BOLD),
-                sub_modifier: Some(Modifier::ITALIC),
+                add_modifiers: modifier(Modifier::Bold),
+                sub_modifiers: modifier(Modifier::Italic),
             }
         );
     }
@@ -642,8 +871,8 @@ mod test {
             style: Style {
                 fg: Some(Color::Red),
                 bg: None,
-                add_modifier: Some(Modifier::BOLD),
-                sub_modifier: None,
+                add_modifiers: modifier(Modifier::Bold),
+                sub_modifiers: None,
             },
         };
 
@@ -653,8 +882,8 @@ mod test {
             style: Style {
                 fg: None,
                 bg: Some(Color::Blue),
-                add_modifier: None,
-                sub_modifier: Some(Modifier::DIM),
+                add_modifiers: None,
+                sub_modifiers: modifier(Modifier::Dim),
             },
         };
 
@@ -664,8 +893,8 @@ mod test {
             style: Style {
                 fg: Some(Color::Cyan),
                 bg: Some(Color::Magenta),
-                add_modifier: Some(Modifier::CROSSED_OUT),
-                sub_modifier: Some(Modifier::ITALIC),
+                add_modifiers: modifier(Modifier::CrossedOut),
+                sub_modifiers: modifier(Modifier::Italic),
             },
         };
 
@@ -677,8 +906,8 @@ mod test {
                 style: Style {
                     fg: Some(Color::Red),
                     bg: Some(Color::Blue),
-                    add_modifier: Some(Modifier::BOLD),
-                    sub_modifier: Some(Modifier::DIM),
+                    add_modifiers: modifier(Modifier::Bold),
+                    sub_modifiers: modifier(Modifier::Dim),
                 },
             }
         );
@@ -691,8 +920,8 @@ mod test {
                 style: Style {
                     fg: Some(Color::Red),
                     bg: Some(Color::Blue),
-                    add_modifier: Some(Modifier::BOLD),
-                    sub_modifier: Some(Modifier::DIM),
+                    add_modifiers: modifier(Modifier::Bold),
+                    sub_modifiers: modifier(Modifier::Dim),
                 },
             }
         );
@@ -705,8 +934,8 @@ mod test {
                 style: Style {
                     fg: Some(Color::Cyan),
                     bg: Some(Color::Magenta),
-                    add_modifier: Some(Modifier::CROSSED_OUT),
-                    sub_modifier: Some(Modifier::ITALIC),
+                    add_modifiers: modifier(Modifier::CrossedOut),
+                    sub_modifiers: modifier(Modifier::Italic),
                 },
             }
         );
